# -*- coding: utf-8 -*-
import scrapy, re, json
from datetime import datetime, date
from heraldoHn.items import NoticiasItem

"""
MEDIO:
El Heraldo, Honduras
USO:
// Si se quiere obtener todas las noticias desde las más actuales hasta las más antiguas. //
scrapy crawl noticias --nolog -s filename=noticias.json
-------------------------------------------------------------------------------------------------
// Si se quiere obtener todas las noticias desde las más actuales hasta una fecha específica. //
scrapy crawl noticias --nolog -s filename=noticias.json -a year=2018 -a month=3 -a day=9
-------------------------------------------------------------------------------------------------
Después será necesario hacer uso del archivo parse_date_files.py para que las noticias contenidas
en noticias.json sean separadas en archivos por fecha.
"""

TAG_RE = re.compile(r'<[^>]+>')
def remove_tags(text):
    return TAG_RE.sub('', text)

LOC = re.compile(r'[A-Z].*?, ?[A-Z].+?\.')


class ImportantData(scrapy.Item):
    CONTINUE_SEARCHING = scrapy.Field()
    LAST_LINK = scrapy.Field()
    page = scrapy.Field()
    section_url = scrapy.Field()
    url = scrapy.Field()


class QuotesSpider(scrapy.Spider):
    name = "noticias"

    def start_requests(self):
        year = getattr(self, "year", None)
        month = getattr(self, "month", None)
        day = getattr(self, "day", None)

        if year is not None and month is not None and day is not None:
            self.stopDate = date(int(year), int(month), int(day))
        else:
            self.stopDate = None

        baseURL = "http://www.elheraldo.hn/"

        sectionList = ["tegucigalpa", "economia", "mundo", "revistas/crimenes",
                       "pais", "sucesos", "deportes", "entretenimiento"]
        # sectionList = ["tegucigalpa"]

        if self.stopDate is None:
            for s in sectionList:
                info = ImportantData()
                info['page'] = 1
                request = scrapy.Request(url=baseURL + s + "/", callback=self.parse)
                request.meta['item'] = info
                yield request

        else:
            for s in sectionList:
                info = ImportantData()
                info['page'] = 1
                info['CONTINUE_SEARCHING'] = False
                request = scrapy.Request(url=baseURL + s + "/", callback=self.parse_with_stop_date)
                request.meta['item'] = info
                yield request


    def parse(self, response):
        print response.url
        searchData = response.meta['item']
        CONTINUE_SEARCHING = True
        linkSet = set()
        if searchData['page'] == 1:
            searchData['section_url'] = response.url
            linkSet = linkSet.union(set(response.xpath('//article[@id="destacada"]/a/@href').extract()))
            linkSet = linkSet.union(set(response.xpath('//aside[@id="mini_sidebar"]/article/a/@href').extract()))
            linkSet = linkSet.union(set(response.xpath('//section[@id="principal"]/article/a/@href').extract()))
            linkSet = linkSet.union(set(response.xpath('//aside[@id="otras_noticias"]/article/a/@href').extract()))
            linkSet = linkSet.union(set(response.xpath('//div[@class="contenedor"]/article/a/@href').extract()))
            linkSet = linkSet.union(set(response.xpath('//article[@class="nobordes"]/div/a/@href').extract()))
            linkSet.remove(searchData['section_url'])

        else:
            linkSet = linkSet.union(set(response.xpath('//div[@class="contenedor"]/article/a/@href').extract()))
            linkSet = linkSet.union(set(response.xpath('//article[@class="nobordes"]/div/a/@href').extract()))
            try:
                linkSet.remove(searchData['section_url'])
            except KeyError:
                pass
            if len(linkSet) <= 0:
                CONTINUE_SEARCHING = False

        for link in linkSet:
            yield scrapy.Request(url=link, callback=self.parse_item)

        if CONTINUE_SEARCHING:
            searchData['page'] += 1
            page = searchData['page']
            url = searchData['section_url']
            request = scrapy.Request(url=url + "?page=" + str(page), callback=self.parse)
            request.meta['item'] = searchData
            yield request


    def parse_with_stop_date(self, response):
        searchData = response.meta['item']
        CONTINUE_SEARCHING = searchData['CONTINUE_SEARCHING']

        if not CONTINUE_SEARCHING:
            if searchData['page'] == 1:
                searchData['section_url'] = response.url
                linkList = response.xpath('//article[@id="destacada"]/a/@href').extract()
                linkList.extend(response.xpath('//aside[@id="mini_sidebar"]/article/a/@href').extract())
                linkList.extend(response.xpath('//section[@id="principal"]/article/a/@href').extract())
                linkList.extend(response.xpath('//aside[@id="otras_noticias"]/article/a/@href').extract())
                linkList.extend(response.xpath('//div[@class="contenedor"]/article/a/@href').extract())
                linkList.extend(response.xpath('//article[@class="nobordes"]/div/a/@href').extract())
                linkList.remove(searchData['section_url'])

            else:
                linkList = response.xpath('//div[@class="contenedor"]/article/a/@href').extract()
                linkList.extend(response.xpath('//article[@class="nobordes"]/div/a/@href').extract())
                try:
                    linkList.remove(searchData['section_url'])
                except KeyError:
                    pass

            newsList = []
            for link in linkList:
                if not link in newsList:
                    newsList.append(link)

            for link in linkList:
                info = ImportantData()
                info['url'] = response.url
                info['page'] = searchData['page']
                info['section_url'] = searchData['section_url']
                if link == linkList[-1]: info['LAST_LINK'] = True
                else: info['LAST_LINK'] = False
                reqst = scrapy.Request(url=link, callback=self.parse_item_with_stop_date)
                reqst.meta['item'] = info
                yield reqst

        else:
            searchData['CONTINUE_SEARCHING'] = False
            searchData['page'] += 1
            page = searchData['page']
            url = searchData['section_url']
            request = scrapy.Request(url=url + "?page=" + str(page), callback=self.parse_with_stop_date)
            request.meta['item'] = searchData
            yield request


    def parse_item(self, response):
        item = NoticiasItem()
        d = response.xpath('//time/text()').extract_first()
        res = remove_tags(response.xpath('//script[@type="application/ld+json"]').extract_first())
        newsData = json.loads(res)

        item['date'] = datetime.strptime(d, '%d.%m.%Y').isoformat("T")
        item['title'] = newsData['headline']

        try:
            topic = newsData['articleSection']
        except:
            topic = None
        item['topic'] = topic

        text = newsData['articleBody']
        if text.find(u'\u00a0') >= 0:
            loc = text[:text.find(u'\u00a0')] + "."
            m = LOC.match(loc)
            if m:
                item['location'] = m.group(0)
                text = text[text.find(u'\u00a0') + 1:]

        item['text'] = text

        item['url'] = response.url

        yield item


    def parse_item_with_stop_date(self, response):
        d = response.xpath('//time/text()').extract_first()
        dt = datetime.strptime(d, '%d.%m.%Y').date()

        if dt >= self.stopDate:
            info = response.meta['item']
            item = NoticiasItem()
            res = remove_tags(response.xpath('//script[@type="application/ld+json"]').extract_first())
            newsData = json.loads(res)

            item['date'] = datetime.strptime(d, '%d.%m.%Y').isoformat("T")
            item['title'] = newsData['headline']

            try:
                topic = newsData['articleSection']
            except:
                topic = None
            item['topic'] = topic

            text = newsData['articleBody']
            if text.find(u'\u00a0') >= 0:
                loc = text[:text.find(u'\u00a0')] + "."
                m = LOC.match(loc)
                if m:
                    item['location'] = m.group(0)
                    text = text[text.find(u'\u00a0')+1:]

            item['text'] = text

            item['url'] = response.url

            yield item

            if info['LAST_LINK']:
                info['CONTINUE_SEARCHING'] = True
                request = scrapy.Request(url=info['url'], callback=self.parse_with_stop_date, dont_filter=True)
                request.meta['item'] = info
                yield request
